// this is the main panel for the Zapitista Tribal Port Scan
/*
--------------------------------------------
Gif graphics and airplane display here			graphicPanel.java
--------------------------------------------

Text Area with scan		tcp box			controlPanel.java
results here			udp box
				Zap box (mixed mode)

minutes	<-- scan interval ||| timer --> timeleft

1 min <-----------Slider-----------> 60 min
--------------------------------------------
target	 	the.domain.scanned			targetPanel.java
--------------------------------------------
Applets only will display a message area...		messagePanel.java
--------------------------------------------
*/

import java.awt.*;

class zapsPanel extends Panel {
	GridBagConstraints gc;
	Font thefont;
	graphicPanel gpanel;
	controlPanel cpanel;
	targetPanel tpanel;
	String target;
	zapsTimer timer;
	private ztps parent_applet;
	// the portScan class uses psf int's UDP=1 and TCP=0
	// ZAP is defined here because it makes no sense in
	// the portscan class - it simply means pick a random protocol...
	public static final int ZAP=2;
	private int current_type=ZAP; // Zap is the default
	private String tcp_descript="use Transmission Control Protocol";
	private String udp_descript="send User Datagram Packets";
	private String zap_descript="choose TCP or UDP randomly";
	private boolean is_applet; // am I a (true) applet, or a frame (false)
	private zapsMessage message;


	zapsPanel(Font thefont, Image img, Image[] airplane,
		MediaTracker tracker, String target,
		Object parent, zapsMessage message, boolean is_applet) {
		// font
		this.thefont=thefont;
		this.message=message; // zapsMessage object
		// hold a ref to the parent container- needed to update message area
		// must cast to ztps if applet,
		if (is_applet) {
			this.parent_applet=(ztps)parent;
		}
		this.is_applet=is_applet;
		// instantiate the panel objects
		gpanel= new graphicPanel(this, img, airplane, tracker);
		cpanel= new controlPanel(this);
		tpanel= new targetPanel(this, target, is_applet);
		gc= new GridBagConstraints();
		gc.insets=new Insets(5,5,5,5);

		// set target
		this.target=target;

		// set the layout
		setLayout(new GridBagLayout());
		setBackground(Color.black);

		// add the panels
		setConstraints(gc,0,0,1,1,100,99);
		gc.anchor=GridBagConstraints.CENTER;
		gc.fill=GridBagConstraints.BOTH;
		add(gpanel,gc);
		setConstraints(gc,0,1,1,1,100,1);
		gc.anchor=GridBagConstraints.CENTER;
		gc.fill=GridBagConstraints.BOTH;
		add(cpanel,gc);
		setConstraints(gc,0,2,1,1,100,1);
		gc.anchor=GridBagConstraints.CENTER;
		gc.fill=GridBagConstraints.BOTH;
		add(tpanel,gc);

		// play animation
		gpanel.playGlider();
		// play the sound if it is an applet - desktop ap version has no sound...
		if (is_applet) {
			parent_applet.play_paper_plane_sound();
		}

		// scan a port to begin with (random ZAP)
		new portScan(target,
			((int)(Math.random()*65536)+1),
			cpanel.ta,
			this,
			((int)(Math.random()*2)), //random type
			message.getString());

		// make a timer
		timer = new zapsTimer(1,this);
	}

	// helper for setting up gridBagconstraints...
	protected static void setConstraints(GridBagConstraints c, int gridx, int gridy,
 int gridwidth, int gridheight, int weightx, int weighty ) {
		c.gridx=gridx;
		c.gridy=gridy;
		c.weightx=weightx;
		c.weighty=weighty;
		c.gridwidth=gridwidth;
		c.gridheight=gridheight;
	}

	// when the scan is done, the portScan will call back here.
	// stop the animation.
	protected void scanDoneCallBack() {
		gpanel.showBanner();
	}

	// called by controlPanel.itemStateChanged(int)
	// user selected a new scanning option
	protected void userSelected(int type) {
		// store the currently selected type of scan
		// for the timer to use
		current_type=type;
		if (is_applet) { // if an applet, update message panel...
			switch (type) {
				case portScan.TCP:
					parent_applet.updateMessagePanel(tcp_descript);
					break;
				case portScan.UDP:
					parent_applet.updateMessagePanel(udp_descript);
					break;
				case ZAP:
					parent_applet.updateMessagePanel(zap_descript);
					break;
				default:
			}
		}
	}

	// called by timer - sends a scan when time is up
	protected void sendScan() {
		int type_used;
		// play animation
		gpanel.playGlider();
		// play the sound if it is an applet - desktop ap version has no sound...
		if (is_applet) {
			parent_applet.play_paper_plane_sound();
		}
		// send the scan
		if (current_type == ZAP) { // if ZAP, choose a random method
			type_used=(int)(Math.random()*2);
		} else {
			type_used=current_type;
		}
		new portScan(target,
			((int)(Math.random()*65536)+1),
			cpanel.ta,
			this,
			type_used,
			message.getString());
		// getting the setting from cpanels getMinutes method.
		updateTimerLab(cpanel.getMinutes());
		// start a new timer
		timer= new zapsTimer(cpanel.getMinutes(), this);
	}

	// updates the controlPanel's (timer) label
	// intended for use by the zapsTimer object
	protected void updateTimerLab(int newminutes) {
		// should add a mutator method to cpanel...
		cpanel.timer_lab.setText(Integer.toString(newminutes));
	}

	// set a new target
	protected void newTarget(String target) {
		this.target=target;
		cpanel.ta.append(target+":\n");
	}

	// return current target
	protected String getTarget() {
		return target;
	}

	// spam
	public void finalize() {
		System.out.println("http://www.thing.net/~rdom/ecd/ecd.html");
	}

	// spam
	public String toString() {
		return "Zapatista Tribal Port Scan, Electronic Disturbance Theater, 2K1. http://www.thing.net/~rdom/ecd/ecd.html http://cadre.sjsu.edu/beestal";
	}
}
